#!/bin/sh
#
# This file was produced by running the Configure script. It holds all the
# definitions figured out by Configure. Should you modify one of these values,
# do not forget to propagate your changes by running "Configure -der". You may
# instead choose to run each of the .SH files by yourself, or "Configure -S".
#

# Package name      : dist
# Source directory  : .
# Configuration time: ma 24 elo    06.59.36 1998
# Configured by     : jhi
# Target system     : osf1 alpha.hut.fi v4.0 878 alpha 

Author=''
Date=''
Header='$Header'
Id='$Id'
Locker='$Locker'
Log='$Log'
Mcc=''
RCSfile=''
Revision=''
Source=''
State=''
afs='false'
ar=''
archobjs=''
awk='/usr/bin/awk'
baserev='3.0'
bash=''
bison=''
byacc=''
c='\c'
cat='/usr/bin/cat'
cf_by='jhi'
cf_email='jhi@iki.fi'
cf_time='ma 24 elo    06.59.36 1998'
chgrp='/usr/bin/chgrp'
chmod='/usr/bin/chmod'
chown='/usr/bin/chown'
comm=''
compress=''
contains='grep'
cp=''
cpio=''
cpp='/usr/bin/cpp'
cpplast=''
cppminus=''
cpprun='/usr/bin/cpp'
cppstdin='/m/fs/work/work/permanent/perl/mc/dist-3.0@70b/cppstdin'
csh=''
d_berknames='define'
d_bsd='undef'
d_eunice='undef'
d_passnames='define'
d_portable='undef'
d_usgnames='undef'
d_xenix='undef'
date='/usr/bin/date'
defeditor='/usr/bin/vi'
echo='/usr/bin/echo'
egrep=''
emacs=''
eunicefix=':'
expr='/usr/bin/expr'
find=''
firstmakefile=''
flex=''
gcc=''
grep='/usr/bin/grep'
groupcat='cat /etc/group'
gzip=''
hint='previous'
hostcat='cat /etc/hosts'
inews='/p/bin/inews'
install='/p/bin/ginstall'
installdir='/p/bin/ginstall -d'
installmansrc='/u/vieraat/vieraat/jhi/Perl/man/man1'
installprivlib='/u/vieraat/vieraat/jhi/Perl/lib/dist'
installscript='/u/vieraat/vieraat/jhi/Perl/bin'
ksh=''
less='/c/bin/less'
line='/usr/bin/line'
lint=''
ln='/usr/bin/ln'
lns='/usr/bin/ln -s'
lp=''
lpr=''
ls=''
mail='/usr/bin/mail'
mailer='/usr/sbin/sendmail'
mailpatches=''
mailx=''
make='/usr/bin/make'
make_set_make=''
manext='1'
mansrc='/u/vieraat/vieraat/jhi/Perl/man/man1'
mansrcexp='/u/vieraat/vieraat/jhi/Perl/man/man1'
mkdep='/m/fs/work/work/permanent/perl/mc/dist-3.0@70b/mkdep'
mkdir='/usr/bin/mkdir'
more='/usr/bin/more'
mv='/usr/bin/mv'
mydomain='.hut.fi'
myhostname='alpha'
myuname='osf1 alpha.hut.fi v4.0 878 alpha '
n=''
nametype='bsd'
notifypatches=''
nroff='/usr/bin/nroff'
orgname='Illuminati'
osname=''
osvers=''
package='dist'
pager='/c/bin/less'
passcat='cat /etc/passwd'
patchlevel='70'
perl='/u/vieraat/vieraat/jhi/Perl/bin/perl'
perlpath='/usr/bin/perl'
pg='/usr/bin/pg'
phostname='hostname'
pmake=''
pr=''
prefix='/u/vieraat/vieraat/jhi/Perl'
prefixexp='/u/vieraat/vieraat/jhi/Perl'
privlib='/u/vieraat/vieraat/jhi/Perl/lib/dist'
privlibexp='/u/vieraat/vieraat/jhi/Perl/lib/dist'
rm='/usr/bin/rm'
rmail='/usr/bin/rmail'
scriptdir='/u/vieraat/vieraat/jhi/Perl/bin'
scriptdirexp='/u/vieraat/vieraat/jhi/Perl/bin'
sed='/usr/bin/sed'
sendmail='/usr/sbin/sendmail'
sh='/bin/sh'
shar=''
sharpbang='#!'
shsharp='true'
sleep=''
smail='smail'
sort='/usr/bin/sort'
spackage='Dist'
spitshell='cat'
src='.'
startperl='#!/usr/bin/perl'
startsh='#!/bin/sh'
submit=''
sysman='/usr/man/man1'
tail=''
tar=''
tbl=''
test='test'
touch='/usr/bin/touch'
tr='/usr/bin/tr'
troff=''
uname='/usr/bin/uname'
uniq='/usr/bin/uniq'
usermail=''
uuname=''
vi='/usr/bin/vi'
zcat='/c/bin/zcat'
zip=''
VERSION=3.0
PATCHLEVEL=70
CONFIG=true
